# --- [ digi_output.py ] --------------------------------------------------------

from tkinter import *
import _thread
import time
import net_io_tools as ni

class digi_output(Checkbutton):
    """ Ein Checkbutton mit einem Label ermöglicht, den Status eines Pins zu ändern.
        Beim Anklicken wird der neue Status gesendet.

        .set_Button() übernimmt den aktuellen Pinstatus
        .get_Button() liefert den aktuellen Status """
    
    def __init__(self, master,
                  portpin = 0,              # Der jeweilige Portpin, der geschaltet werden soll
                  label_text = "",
                  font = ("Courier", 10, "bold"),
                  **kw):
        
        Checkbutton.__init__(self, master, text = label_text,
                             width = 12,
                             font = font,
                             command = self.__set_Pinstate,
                             anchor = "w",
                             **kw)

        self.__portpin = portpin
        self.__variable = IntVar()
        self.__variable.set(0)
        self.config(variable = self.__variable)

        self.set_Button(0)
        
    def set_Button(self, value):

        if type(value) == str:
            value = int(value)
            
        if value == 0:
            relief = "raised"
        else:
            value = 1
            relief = "sunken"

        self.configure(relief = relief)
        self.__variable.set(value)
        
    def get_Button(self):
        return self.__variable.get()

    def __set_Pinstate(self):
        self.set_Button(self.__variable.get())
        try:
            ni.Open_Telnet()
            ni.Set_Port_State(self.__portpin, self.__variable.get())
            ni.Close_Telnet()
            
        except:
            print("Telnet-Error")
        
if __name__ == "__main__":

    def animate():
        i = 0
        while True:
            time.sleep(1)
            j = (i % 2)
            i += 1
        
            digi1.set_Button(j)
            #digi2.set_Button(j)
            

            print(digi1.get_Button())

    top = Tk()
    frm1 = LabelFrame(top, text = "test")
    frm1.grid(column = 0, row = 0, padx = 5, pady = 3)

    digi1 = digi_output(frm1, label_text = "Output 1:", font = ("Courier", 8), portpin = 0)
    digi1.grid(column = 0, row = 0, padx = 5, pady = 3)

    digi2 = digi_output(frm1, label_text = "Output 2:", portpin = 1)
    digi2.grid(column = 1, row = 1, padx = 5, pady = 3)


    _thread.start_new_thread(animate,())

    top.mainloop()

# --- [ eof ] ------------------------------------------------------------------
