# --- [ lcd_outputs.py ] --------------------------------------------------------

from tkinter import *
import _thread
import time
import net_io_tools as ni

class lcd_output(Frame):
    """ In einen Frame als unsichtbaren Container werden 1 Label, 1 Entry-Feld
        und zwei Buttons eingebaut.
        
        deactivate() deaktiviert die Eingabefelder (wenn kein LCD vorhanden ist) """
    
    def __init__(self, master,
                 font = ("Courier", 10, "bold"),
                 lcd_row = 1,
                  **kw):
        Frame.__init__(self, master,  **kw)
        self.grid(column = 0, row = (lcd_row - 1))

        self.__lbl = Label(self, text = "LCD row " + str(lcd_row) + ":", width = 10, anchor = "e" ,font = font )
        self.__lbl.grid(column = 0, row = 0, padx = 5, pady = 3, sticky = "e")
        
        self.__entry = Entry(self, width = 16 , font = font)
        self.__entry.grid(column = 1, row = 0, padx = 5, pady = 2, sticky = "nse")
        self.__variable = StringVar()
        self.__entry.config(textvariable = self.__variable)
        self.__entry.bind(sequence ='<Return>', func= self.__send_row) 

        self.__btn1 = Button(self, width = 10, text = "send row", font = font, command = self.__send_row)
        self.__btn1.grid(column = 2, row = 0, padx = 5, pady = 3, sticky = "e")

        self.__btn2 = Button(self, width = 10, text = "clr row", font = font, command = self.__clr_row)
        self.__btn2.grid(column = 3, row = 0, padx = 5, pady = 3, sticky = "e")

        self.__lcd_row = lcd_row
        self.__lcd_found = True
        
    def __clr_row(self):
        self.__variable.set("")
        if self.__lcd_found:
            try:
                ni.Open_Telnet()
                ni.Clear_LCD(self.__lcd_row)
                ni.Close_Telnet()
            except:
                print("Telnet_error")
                
    def __send_row(self, event = None):
        if self.__lcd_found:
            try:
                ni.Open_Telnet()
                ni.Write_LCD(self.__lcd_row, self.__variable.get())
                ni.Close_Telnet()
            except:
                print("Telnet-error")
        
    def deactivate(self):
        self.__variable.set("no lcd found")
        self.__entry.configure(state  = "disabled")
        self.__btn1.configure(text = "-")
        self.__btn2.configure(text = "-")
        self.__lcd_found = False
        

if __name__ == "__main__":

    def animate():
        
        time.sleep(1)           # schaltet nach 1 Sekunde lcd1 deaktiv
        lcd1.deactivate()

        while True:
            time.sleep(1)

    top = Tk()
    frm1 = LabelFrame(top, text = "test")
    frm1.grid(column = 0, row = 0, padx = 5, pady = 3)

    lcd1 = lcd_output(frm1, font = ("Courier", 8), lcd_row = 1)
    lcd1.grid(column = 0, row = 0)

    lcd2 = lcd_output(frm1, lcd_row = 2)
    lcd2.grid(column = 1, row = 1)

    _thread.start_new_thread(animate,())

    top.mainloop()

# --- [ eof ] ------------------------------------------------------------------
