# ---- [net_io_gui.py ] -------------------------------------------------------
#
#       23.11.12

import _thread
import time
import tkinter      as tk
import net_io_tools as ni
import bargraph     as bg

from digi_inputs   import *
from analog_inputs import *
from digi_outputs  import *
from lcd_outputs   import *

# ------------------------------------------------------------------------------
# ein eigenständiger Thread, der in regelmäßigen Abständen den AVR-NET-IO anfunkt,
# die Daten abholt und die Anzeige im Hauptfenster aktualisiert.

def Timer_Thread():
    global telnet_error                 # Fehler beim Telnet-Zugriff
    
    while not telnet_error:
        ni.Open_Telnet()                # die Verbindung zum Server öffnen

        adc = ni.Get_Adcs()             # ADC's auslesen
                      
        ana_in1.set_Value(adc[0])       # die numerische Anzeige (neben dem Text "ADCx") aktualisieren
        ana_in2.set_Value(adc[1])
        ana_in3.set_Value(adc[2])
        ana_in4.set_Value(adc[3])
        
        bg1.draw(adc[0])                # die Anzeige der Bargraphen aktualisieren
        bg2.draw(adc[1])
        bg3.draw(adc[2])
        bg4.draw(adc[3])
        
        pinstate = ni.Get_Pin_State()   # Den Status der digitalen Eingänge einlesen
        
        digi_in1.set_Button(pinstate[0])
        digi_in2.set_Button(pinstate[1])
        digi_in3.set_Button(pinstate[2])
        digi_in4.set_Button(pinstate[3])

        Refresh_Digi_Ports()            # liest den Status der Ausgänge ein

        ni.Close_Telnet()               # Verbindung zum Telnetserver kappen
        
        sleeptime = interval.get()      # das aktuell gewählte Intervall aus der Spinbox holen

        while sleeptime > 0:            # Wartezeit
            time.sleep(1)               # wartet immer nur für 1 Sekunde
            sleeptime -= 1              # 

            next_interval.set(sleeptime)# Anzeige der Wartezeit bis zur nächsten Messung aktualisieren

# ------------------------------------------
# liest den Status aller Ports ein und weist den Wert
# den Variablen zu, die den Checkbutton steuern
def Refresh_Digi_Ports():

    l = ni.Get_Port_State()
   
    digi_out1.set_Button(l[0])
    digi_out2.set_Button(l[1])
    digi_out3.set_Button(l[2])
    digi_out4.set_Button(l[3])
    digi_out5.set_Button(l[4])
    digi_out6.set_Button(l[5])
    digi_out7.set_Button(l[6])
    digi_out8.set_Button(l[7])


# -------------------------------------------
# Der "Beenden"- Button beendet die Anwendung
def ende():
    top.destroy()
    
# -------------------------------------------
# Wird aufgerufen, wenn der Button "I" angeklickt wurde
# gibt eine Messagebox mit den Hardwareinformationen des AVR-NET-IO aus

# Die Messagebox führt zu Programmende, wenn nicht über idle ausgeführt wird !
# Ausserdem zählt die Zeit nicht weiter ...
# def avrnetio_version():
#    tk.messagebox.showinfo("AVR-NET-IO Info", netio_message[1][0] + ' - '
#                     + netio_message[1][1] + ' - ' + netio_message[1][2])
#
#
# diese Variante funktioniert auch dann, wenn nicht aus Idle heraus gestartet wird
# und blockiert nicht die Ausführung des anderen Threads
def avrnetio_version():

    w = tk.Toplevel()       # ein unabhängiges Toplevel-Window öffnen
    w.resizable(width = False, height = False) # Größe nicht veränderbar
    w.title("AVR-NET-IO Info")
    f = tk.Label(w, width = 30, height = 5, font = ft,  text = (netio_message[1][0] + '\n' +
                     netio_message[1][1] + '\n' + netio_message[1][2]), anchor = "center")
    f.grid(column = 0, row = 0, sticky = "ewns")
    w.focus_set()           # focus auf das Fenster
    w.grab_set()            # alle Events werden nur nach w durchgereicht,
                            # das drunterliegende Fenster bleibt inaktiv, wird aber aktualisiert
    w.transient(top)        # das Fenster bleibt oberhalb von Top
#   w.wait_window(w)        # warten bis zum Beenden des Fensters, aber ...
                            # ... die Zeit zählt dann nicht weiter
    w.mainloop()            # warten bis zum Beenden des Fensters


# --- Das Mainwindow -----------------------------------------------------
top = tk.Tk()
top.resizable(width = False, height = False)    # die Fenstergröße unveränderbar machen


# --- Messintervall --------------------------
interval = tk.IntVar()
interval.set(10)                 # Startwert für das Messintervall
next_interval = tk.IntVar()
telnet_error = False

# --- Fonts ---------------------------------------

ft = ("Courier",10,"bold" )     # Standard Schrift
#ft = ("LucidiaTypeWriter", 10, "bold")
fs = ("LucidiaTypeWriter", 8)   # Beschriftung der Frames und Bargraphen

# --- Jetzt folgt der Aufbau des Windows
#     Mit dem Grid-Manager wird die Seite analog wie eine HTML-Tabelle aufgebaut.
#     Im Hauptfenster Top werden Frames durch Angabe ihrer Position über row und column definiert
#     Innerhalb eines Frames werden die Widgets in gleicher Form in Tabellen-"Zellen" plaziert
#     Die Breite und Höhe einer jeden Zelle richtet sich nach dem größten Element, dass die
#     zugehörige row / column aufnehmen muss.


# --- Frame 1, für die 4 digitalen Eingänge -------

frame1 = tk.LabelFrame(top, padx = 5, pady = 5, text = "Read Digital Inputs", relief = "groove", font = fs)
frame1.grid(column = 0, row = 0, columnspan = 2, padx = 5, pady = 5)

digi_in1 = digi_input(frame1, label_text = "Input 1:")
digi_in2 = digi_input(frame1, label_text = "Input 2:")
digi_in3 = digi_input(frame1, label_text = "Input 3:")
digi_in4 = digi_input(frame1, label_text = "Input 4:")

digi_in1.grid(column = 0, row = 0, padx = 5, pady = 3, sticky = "w")
digi_in2.grid(column = 0, row = 1, padx = 5, pady = 3, sticky = "w")
digi_in3.grid(column = 0, row = 2, padx = 5, pady = 3, sticky = "w")
digi_in4.grid(column = 0, row = 3, padx = 5, pady = 3, sticky = "w")

# --- FRAME 2, für die 10 digitalen Ausgänge --------

frame2 = tk.LabelFrame(top, padx = 5, pady = 5, text = "Set Outputs", relief = "groove", font = fs)
frame2.grid(column = 2, row = 0, columnspan = 2, padx = 5, pady = 5, sticky = "nsew")

digi_out1 = digi_output(frame2, label_text = "Output 1", portpin = 1, font = ft)
digi_out2 = digi_output(frame2, label_text = "Output 2", portpin = 2, font = ft)
digi_out3 = digi_output(frame2, label_text = "Output 3", portpin = 3, font = ft)
digi_out4 = digi_output(frame2, label_text = "Output 4", portpin = 4, font = ft)
digi_out5 = digi_output(frame2, label_text = "Output 5", portpin = 5, font = ft)
digi_out6 = digi_output(frame2, label_text = "Output 6", portpin = 6, font = ft)
digi_out7 = digi_output(frame2, label_text = "Output 7", portpin = 7, font = ft)
digi_out8 = digi_output(frame2, label_text = "Output 8", portpin = 8, font = ft)

digi_out1.grid(column = 0, row = 0, padx = 5, pady = 3, sticky = "w")
digi_out2.grid(column = 0, row = 1, padx = 5, pady = 3, sticky = "w")
digi_out3.grid(column = 0, row = 2, padx = 5, pady = 3, sticky = "w")
digi_out4.grid(column = 0, row = 3, padx = 5, pady = 3, sticky = "w")
digi_out5.grid(column = 1, row = 0, padx = 5, pady = 3, sticky = "w")
digi_out6.grid(column = 1, row = 1, padx = 5, pady = 3, sticky = "w")
digi_out7.grid(column = 1, row = 2, padx = 5, pady = 3, sticky = "w")
digi_out8.grid(column = 1, row = 3, padx = 5, pady = 3, sticky = "w")

# --- Frame 3, für die 4 analogen Eingänge mit digitaler und analoger Darstellung der Messwerte

frame3 = tk.LabelFrame(top, padx = 5, pady = 5, text = "Read 10-Bit Analog Inputs", relief = "groove", font = fs)
frame3.grid(column = 0, row = 2, columnspan = 4, padx = 5, pady = 5, sticky = "nsew")

ana_in1 = analog_input(frame3, label_text = "ADC 1:", font = ft)
ana_in2 = analog_input(frame3, label_text = "ADC 2:", font = ft)
ana_in3 = analog_input(frame3, label_text = "ADC 3:", font = ft)
ana_in4 = analog_input(frame3, label_text = "ADC 4:", font = ft)

ana_in1.grid(column = 0, row = 0, padx = 5, pady = 3, sticky = "w")
ana_in2.grid(column = 0, row = 1, padx = 5, pady = 3, sticky = "w")
ana_in3.grid(column = 0, row = 2, padx = 5, pady = 3, sticky = "w")
ana_in4.grid(column = 0, row = 3, padx = 5, pady = 3, sticky = "w")

Space39 = tk.Label(frame3, width = 0)  # Platz-/Abstandshalter
Space39.grid(column = 4, row = 0, padx= 3, pady = 3, sticky = "ew")

bg1 = bg.BarGraph(frame3, width = 256, end_value = 1023, font = ft, label = (5, "Va ", " mV"))
bg2 = bg.BarGraph(frame3, width = 256, end_value = 1023, font = ft, label = (5, "Vb ", " mV"))
bg3 = bg.BarGraph(frame3, width = 256, end_value = 1023, font = ft, label = (5, "Vc ", " mV"))
bg4 = bg.BarGraph(frame3, width = 256, end_value = 1023, font = ft, label = (5, "Vd ", " mV"))


bg1.grid(column = 5, row = 0, padx = 5, pady = 3, sticky = "ne")
bg2.grid(column = 5, row = 1, padx = 5, pady = 3, sticky = "ne")
bg3.grid(column = 5, row = 2, padx = 5, pady = 3, sticky = "ne")
bg4.grid(column = 5, row = 3, padx = 5, pady = 3, sticky = "ne")

# --- Frame 4 - LCD-Eingabezeilen und Buttons zum Senden / Löschen ------

frame4 = tk.LabelFrame(top, padx = 5, pady = 5, text = "Set LCD Output", relief = "groove", font = fs)
frame4.grid(column = 0, row = 3, columnspan = 4, padx = 5, pady = 5, sticky = "nsew")

lcd1 = lcd_output(frame4, lcd_row = 1)
lcd2 = lcd_output(frame4, lcd_row = 2)
lcd1.grid(column = 0, row = 0)
lcd2.grid(column = 0, row = 1)

# -- Frame 5 ---- Auswahl Messintervall, Info und Beenden ---------------

frame5 = tk.LabelFrame(top, padx = 5, pady = 5, relief = "groove", font = fs)
frame5.grid(column = 0, row = 4, columnspan = 4, rowspan = 1, padx = 5, pady = 5, sticky = "new")

Label51 = tk.Label(frame5, text = "Intervall:" , padx = 3, pady = 3, width = 10, font = ft, anchor = "e")
Label51.grid(column = 0, row = 0, padx= 4, pady = 5, sticky = "w")

spinbtn = tk.Spinbox(frame5, font = ft, width = 4, textvariable = interval, from_ = 1, to = 120, state = "readonly")
spinbtn.grid(column = 1, row = 0, padx = 4, pady = 3, sticky = "ew")

Button51 = tk.Label(frame5, text = "[Sek] next in :",  font = ft, width = 14, anchor ="w")
Button51.grid(column = 2, row = 0, padx= 3, pady = 3, sticky = "ew")

Label51 = tk.Label(frame5, textvariable = next_interval,  font = ft, width = 3, anchor ="center")
Label51.grid(column = 3, row = 0, padx= 1, pady = 3, sticky = "ew")

Button52 = tk.Button(frame5, text = "I", font = ft, width = 1, anchor = "center", command = avrnetio_version)
Button52.grid(column = 4, row = 0, padx= 5, pady = 4, sticky = "e")

Button53 = tk.Button(frame5, text = "Quit",  font = ft, width = 10, anchor = "center", command = ende)
Button53.grid(column = 5, row = 0, padx= 5, pady = 3, sticky = "e")

# --- nun ist das Fenster fertig

netio_message = ni.Start_Telnet()       # schauen ob die Verbindung zum Server steht

if len(netio_message[1][2]) == 0:
    telnet_error = True
    # nach tk.messagebox erhalten die Entry-Felder keinen focus.
    # erst nach einem Aufruf von avrnetio_version() ist das wieder möglich
    # daher wird die eingebaute messagebox nicht mehr verwendet 
    #tk.messagebox.showwarning(message = "AVR-NET-IO nicht gefunden")
    avrnetio_version()
       
top.title("AVR-NET-IO - " + netio_message[0]) # die Verbindungsdaten in Fenstertitel aufnehmen

if not telnet_error:
    ni.Open_Telnet()                    # die Verbindung zum Server öffnen, um das LCD auf Präsenz zu testen

    if ni.Init_LCD():
        LCD_OK = True
    
    else:                               # Bei fehlendem LCD treten nach der erfolglosen Initialsierung Probleme auf
        ni.Close_Telnet()               # Als Workaround die Verbindung schließen
                                        # um Fehlerhafte Daten in Get_Port_State() zu vermeiden

        lcd1.deactivate()               # Eingabefelder und Buttons bei fehlendem LCD deaktivieren
        lcd2.deactivate()

        ni.Open_Telnet()                # Workaround - Verbindung wieder öffnen
    
    Refresh_Digi_Ports()                # den Status die digitalen Ausgänge auslesen,
                                        # und die Anzeige aktualisieren

    ni.Close_Telnet()                   # die Verbindung zum Server schließen


_thread.start_new_thread(Timer_Thread,())# den Thread starten, der die Messungen anstößt und die Messwerte aktualisiert

top.mainloop()

# --- [ eof ] ----------------------------------------------------------------------------
